\NeedsTeXFormat{LaTeX2e}[1995/12/01]
\ProvidesClass{thud}[2014/01/13]

\RequirePackage{kvoptions}
\SetupKeyvalOptions{prefix={\@currname @opt@}}

\DeclareBoolOption{draft}
\DeclareComplementaryOption{final}{draft}
\DeclareStringOption{target}
\DeclareStringOption[italian]{babel}

\DeclareDefaultOption{\PassOptionsToClass{\CurrentOption}{book}}
\ProcessKeyvalOptions{\@currname}

\def\thud@save#1{\expandafter\let\csname thud@save@#1\expandafter\endcsname\csname#1\endcsname}
\def\thud@restore#1{\expandafter\let\csname#1\expandafter\endcsname\csname thud@save@#1\endcsname}
\def\@ifplus#1{\@ifnextchar+{\@firstoftwo{#1}}}

\ifthud@opt@draft\PassOptionsToClass{draft}{book}\fi

\def\thud@tmpcmd{bach}
\ifx\thud@opt@target\thud@tmpcmd
    \chardef\thud@target\z@\relax
\else
    \def\thud@tmpcmd{mst}
    \ifx\thud@opt@target\thud@tmpcmd
        \chardef\thud@target\@ne\relax
    \else
        \def\thud@tmpcmd{phd}
        \ifx\thud@opt@target\thud@tmpcmd
            \chardef\thud@target\tw@\relax
        \else
            \ClassError{\@currname}{Invalid value for the \protect"target\protect" option}
                {Specify one of \protect"bach\protect", \protect"mst\protect" or \protect"phd\protect"}
        \fi
    \fi
\fi
\newif\ifthud@phd
\ifnum\thud@target=\tw@\relax\thud@phdtrue\fi

\LoadClass[onecolumn,twoside,\ifthud@phd 10\else 11\fi pt]{book}

\def\@author{\ClassError{\@currname}{No \string\author\space given}{Provide an \string\author}}

\RequirePackage{setspace}
\RequirePackage{graphicx}
\RequirePackage[\thud@opt@babel]{babel}
\RequirePackage[\ifthud@phd papersize={160mm,235mm}\else a4paper\fi]{geometry}

\geometry{%
    centering,%
    bindingoffset=.025\paperwidth,%
    outer=.075\paperwidth,%
    nofoot,nomarginpar,%
    vdivide={.08\paperheight,*,.06\paperheight}%
}

\addto\captionsenglish{
    \def\abstractname{Abstract}
    \def\acknowledgementsname{Acknowledgements}
    \def\chairname{Chair}
    \def\conclusionsname{Conclusions}
    \def\cosupervisorname{Co-Supervisor}
    \def\cosupervisorsname{Co-Supervisors}
    \def\indexname{Index}
    \def\introductionname{Introduction}
    \def\prefacename{Preface}
    \def\refereename{Referee}
    \def\refereesname{Referees}
    \def\summaryname{Summary}
    \def\supervisorname{Supervisor}
    \def\supervisorsname{Supervisors}
    \def\tutorname{Tutor}
    \def\tutorsname{Tutors}
    \def\addressname{Author's Address}
    \def\candidatename{Candidate}
    \def\candidatesname{Candidates}
    \def\emailname{Author's E-mail}
    \def\homepagename{Author's Web Page}
    \def\telephonename{Author's Telephone Number}
    \def\datename{Academic Year}
    \def\thesisname{\ifcase\thud@target Bachelor\space\or Master\space\or Ph.D.~\fi Thesis}
}
\addto\captionsitalian{
    \def\abstractname{Sommario}
    \def\acknowledgementsname{Ringraziamenti}
    \def\chairname{Direttore}
    \def\conclusionsname{Conclusioni}
    \def\cosupervisorname{Co-relatore}
    \def\cosupervisorsname{Co-relatori}
    \def\indexname{Indice analitico}
    \def\introductionname{Introduzione}
    \def\prefacename{Prefazione}
    \def\refereename{Controrelatore}
    \def\refereesname{Controrelatori}
    \def\summaryname{Riassunto}
    \def\supervisorname{Relatore}
    \def\supervisorsname{Relatori}
    \def\tutorname{Tutor aziendale}
    \def\tutorsname{Tutor aziendali}
    \def\addressname{Indirizzo dell'autore}
    \def\candidatename{Candidato}
    \def\candidatesname{Candidati}
    \def\datename{Anno accademico}
    \def\emailname{Posta elettronica dell'autore}
    \def\homepagename{Pagina web dell'autore}
    \def\telephonename{Numero telefonico dell'autore}
    \def\thesisname{Tesi di \ifcase\thud@target Laurea\or Laurea Magistrale\or Dottorato\fi}
}

\ifthud@phd\let\thud@spacing\singlespacing\else\def\thud@spacing{\setstretch{1.3}}\fi

\def\thud@newpagestyle#1#2{
    \@namedef{ps@#1}{
        \let\@oddfoot\@empty
        \let\@evenfoot\@empty
        \let\@mkboth\markboth
        \expandafter\let\expandafter\ps@plain\csname thud@chps@#1\endcsname%
        \@nameuse{thud@chs@#1}%
        \@nameuse{thud@pts@#1}%
        #2%
    }
}

\def\chaptermark#1{\ifnum\c@secnumdepth>\m@ne\relax\def\thud@tmpcmd{\thechapter. \ #1}\else\let\thud@tmpcmd\@empty\fi\markboth{\thud@tmpcmd}{\thud@tmpcmd}}
\def\sectionmark#1{\markright{\ifnum\c@secnumdepth>\z@\relax\thesection. \ \fi#1}}

\def\thud@headruledbox#1#2#3{\underline{\makebox[\textwidth]{%
    {\small#1\bfseries#2\hfill#3}\rule[-0.5ex]{0pt}{1ex}%
}}}
\def\thud@defheads#1{%
    \def\@evenhead{\thud@headruledbox{#1}\thepage\leftmark}%
    \def\@oddhead{\thud@headruledbox{#1}\rightmark\thepage}%
}

\let\ps@bookplain\ps@plain
\let\thud@chps@plain\ps@empty
\let\thud@chps@big\ps@empty
\def\thud@chps@sfbig{
    \def\@evenhead{\rule[-0.5ex]{\textwidth}{4\p@}}
    \def\@oddhead{\rule[-0.5ex]{\textwidth}{4\p@}}
}

\thud@newpagestyle{plain}{\thud@defheads\relax}
\thud@newpagestyle{big}{\thud@defheads\relax}
\thud@newpagestyle{sfbig}{\thud@defheads\sffamily}

\def\thud@declgigantic{\DeclareFontShape{T1}{cmr}{ch}{n}{<-> cmr17}{}}
\def\thud@gigantic{\fontsize{99.45}{100\p@}\usefont{T1}{cmr}{ch}{n}}

\thud@save{@makechapterhead}
\thud@save{@makeschapterhead}
\long\def\thud@mkchaphdstub#1#2#3{%
    {\parindent\z@\relax
    \raggedleft\normalfont
    \vspace*{30\p@}%
    #1%% Either \iftrue or \iffalse
        \ifnum\c@secnumdepth>\m@ne\relax
            \if@mainmatter
                {\thud@gigantic\thechapter}\Huge\par\nobreak\vskip1ex\relax%
            \fi%
        \fi%
    \fi%
    \interlinepenalty\@M\relax%
    \Huge\bfseries#2#3\par\nobreak%
    \vskip40\p@\relax}%
}
\long\def\thud@defmakechapterhead#1{
    \thud@declgigantic%
    \def\@makechapterhead##1{\expandafter\thud@mkchaphdstub\noexpand\iftrue{#1}{##1}}
    \def\@makeschapterhead##1{\expandafter\thud@mkchaphdstub\noexpand\iffalse{#1}{##1}}
}

\def\thud@chs@plain{%
    \thud@restore{@makechapterhead}
    \thud@restore{@makeschapterhead}
}
\def\thud@chs@big{\thud@defmakechapterhead\relax}
\def\thud@chs@sfbig{\thud@defmakechapterhead\sffamily}

\thud@save{@part}
\thud@save{@spart}
\long\def\thud@spartstub#1#2#3#4{
    {\raggedleft
    \interlinepenalty\@M\relax
    \normalfont%
    #1%% Either \iftrue or \iffalse
        \ifnum\c@secnumdepth>-2\relax
            \if@mainmatter
                {\thud@gigantic\thepart\par\vskip20\p@\relax\hrule\vskip20\p@\relax}%
            \fi%
        \fi%
    \else%
        \hrule\vskip20\p@\relax%
    \fi%
    \Huge\bfseries#2\par}%
\begin{figure}[h]
\centering
\includegraphics[width=1\textwidth]{xkcd/#3}
\caption*{[xkcd.org] #4}
\end{figure}
    \@endpart%
}
\def\thud@defpartstub#1{
    \def\@part[##1]##2[##3][##4]{
        \ifnum\c@secnumdepth>-2\relax
            \refstepcounter{part}%
            \addcontentsline{toc}{part}{\thepart\hspace{1em}##1}%
        \else
            \addcontentsline{toc}{part}{##1}%
        \fi%
        \markboth{}{}%
        \expandafter\thud@spartstub\noexpand\iftrue{#1##2}{##3}{##4}%
    }
    \def\@spart##1{
        \expandafter\thud@spartstub\noexpand\iffalse{#1##1}{}{}%
    }
}

\def\thud@pts@plain{%
    \thud@restore{@part}
    \thud@restore{@spart}
}
\def\thud@pts@big{%
    \thud@declgigantic%
    \thud@defpartstub\relax
}
\def\thud@pts@sfbig{%
    \thud@declgigantic%
    \thud@defpartstub\sffamily
}

\def\thud@defentry@b#1#2#3{
    \expandafter\let\csname thud@#3\endcsname\@empty
    \expandafter#2#1%% Either \iftrue or \ifthud@phd
    	\@namedef{#3}##1{\expandafter\gdef\csname thud@#3\endcsname{##1}}
    \else
        \@namedef{#3}##1{\ClassWarning{\@currname}{\expandafter\string\csname#3\endcsname\space in non-PhD thesis}}
    \fi%
}
\def\thud@defentry@a#1{\@ifplus{\expandafter\thud@defentry@b\noexpand\ifthud@phd{#1}}{\expandafter\thud@defentry@b\noexpand\iftrue{#1}}}
\def\thud@defentry{\@ifstar{\thud@defentry@a\long}{\thud@defentry@a\relax}}
\thud@defentry{address}
\thud@defentry{chair}
\thud@defentry{cosupervisor}
\thud@defentry+{email}
\thud@defentry+{homepage}
\thud@defentry+{phdnumber}
\thud@defentry{referee}
\thud@defentry*+{rights}
\thud@defentry{supervisor}
\thud@defentry+{telephone}
\thud@defentry{tutor}

\def\printindex{\InputIfFileExists{\jobname.ind}{}{\ClassWarningNoLine{\@currname}{No file \jobname.ind.}}}

\newif\ifthud@endpart
\def\maybeblank{\ifthud@endpart\else\ifodd\c@page\@endpart\fi\fi\global\thud@endpartfalse}
\g@addto@macro\@endpart{\global\thud@endparttrue}
\def\thud@patchMatter#1{%
    \thud@save{#1matter}
    \@namedef{#1matter}{%
        \thud@save{cleardoublepage}
        \def\cleardoublepage{\maybeblank\thud@endparttrue}
        \@nameuse{thud@save@#1matter}
        \thud@restore{cleardoublepage}
    }
}
\thud@patchMatter{main}
\thud@patchMatter{back}
\thud@save{part}
\def\part{\maybeblank\thud@save@part}

\newcounter{thud@secnumdepth}
\newif\ifthud@unnmkchp
\thud@save{chapter}
\def\chapter{\maybeblank\@ifplus{\thud@unnmkchptrue\thud@save@chapter}{\thud@unnmkchpfalse\thud@save@chapter}}
\thud@save{@chapter}
\def\@chapter[#1]#2{%
    \ifthud@unnmkchp\c@thud@secnumdepth=\c@secnumdepth\relax\c@secnumdepth=-100\relax\fi%
    \thud@save@@chapter[#1]{#2}%
    \ifthud@unnmkchp\c@secnumdepth=\c@thud@secnumdepth\relax\fi%
}
\thud@save{@schapter}
\def\@schapter#1{\thud@save@@schapter{#1\ifthud@unnmkchp\@mkboth{#1}{#1}\fi}}

\def\abstract{\chapter+*{\abstractname}}
\def\acknowledgements{\chapter+*{\acknowledgementsname}}
\def\summary{\chapter+*{\summaryname}}

\newenvironment{titlelike}[1]{
    \cleardoublepage\thispagestyle{empty}%
    \def\thud@titlelike{#1}%
    \vspace*{\z@\@plus.3fil}%
    \begin{\thud@titlelike}%
}{
    \end{\thud@titlelike}%
    \vspace*{\z@\@plus.6fil}%
    \newpage\thispagestyle{empty}%
}
\newenvironment{approval}{\begin{titlelike}{center}}{\end{titlelike}}
\newenvironment{dedication}{\begin{titlelike}{flushright}}{\end{titlelike}}

\def\thud@patchNoUc#1{
    \thud@save{#1}
    \@namedef{#1}{
    	\thud@save{MakeUppercase}
        \let\MakeUppercase\@iden
        \@nameuse{thud@save@#1}%
        \thud@restore{MakeUppercase}
    }
}
\def\thud@patchNoUcI#1{%
    \thud@save{#1}
    \@namedef{#1}##1{
    	\thud@save{MakeUppercase}
        \let\MakeUppercase\@iden
        \@nameuse{thud@save@#1}{##1}%
        \thud@restore{MakeUppercase}
    }
}
\thud@patchNoUc{tableofcontents}
\thud@patchNoUc{listoffigures}
\thud@patchNoUc{listoftables}
\thud@patchNoUc{theindex}
\thud@patchNoUcI{thebibliography}

\thud@save{@starttoc}
\def\@starttoc#1{\singlespacing\thud@save@@starttoc{#1}\thud@spacing}

\gdef\thud@address{%
    \ifthud@phd Dipartimento di Matematica e Informatica\\\fi%
    Universit\`a degli Studi di Udine\\
    Via delle Scienze, 206\\
    33100 Udine --- Italia
}
\gdef\thud@university{%
    Universit\`a degli Studi di Udine\\[.5em]%
    Dipartimento di Matematica e Informatica\\[.5em]%
    \ifcase\thud@target Corso di Laurea\or Corso di Laurea Magistrale\or Dottorato di Ricerca\fi\space in Informatica%
}

\def\thud@showentryA{\@ifstar{\expandafter\thud@showentryA@\noexpand\iffalse}{\expandafter\thud@showentryA@\noexpand\iftrue}}
\def\thud@showentryA@#1#2{%
    \expandafter\ifx\csname thud@#2\endcsname\@empty\else%
        \def\thud@varname{\@nameuse{#2name}}%
        #1%% Either \iftrue or \iffalse
            \def\and{\\[.2ex]\gdef\thud@varname{\@nameuse{#2sname}}}%
        \else%
            \let\and\@undefined
        \fi%
        \begin{lrbox}{\thud@tmpbox}\parbox[t]{\linewidth}{\@nameuse{thud@#2}}\end{lrbox}%
        \textsc{\thud@varname:}\\[1ex]%
        \usebox{\thud@tmpbox}\\[2em]\null%
    \fi%
}

\def\thud@showentryB{\@ifnextchar[\thud@showentryB@{\thud@showentryB@[\relax]}}
\def\thud@showentryB@[#1]#2{%
    \expandafter\ifx\csname thud@#2\endcsname\@empty\else%
        \noindent\@nameuse{#2name}:\quad\begin{tabular}[t]{@{}l@{}}#1\@nameuse{thud@#2}\end{tabular}\\[2em]\null%
    \fi%
}

\def\maketitle{
    \begin{titlepage}%
        \newsavebox\thud@tmpbox
        \noindent\makebox[\z@][l]{\raisebox{-.99\textheight}[\z@][\z@]{\includegraphics[height=\textheight]{UdineLogo}}}
        \begin{minipage}[t][\textheight][s]{.99\linewidth}%
            \let\footnotesize\small
            \let\footnoterule\relax
            {\Large\textsc{\thud@university}}
            \vfill%
            \begin{center}%
                {\large\textsc{\thesisname\ifx\thud@phdnumber\@empty\else: \thud@phdnumber\fi}\par}
                \vskip 4em\relax%
                {\huge\bfseries\@title\par}
                \vskip 5em\@plus1fill\relax%
                {\large%
                    \parbox[t]{.4\linewidth}{%
                        \let\thud@candidate\@author
                        \thud@showentryA{candidate}%
                    }%
                    \quad%
                    \parbox[t]{.4\linewidth}{%
                        \thud@showentryA{supervisor}%
                        \thud@showentryA{cosupervisor}%
                        \thud@showentryA{tutor}%
                        \thud@showentryA{referee}%
                        \thud@showentryA*{chair}%
                    }%
                }
                \par\vskip 1em\@plus1fill\relax%
                {\large\ifx\undefined\datename\else\datename~{}\fi\@date}\par
            \end{center}
            \ifthud@phd\else
                \ifx\thud@address\@empty\else
                    \par\vfil\begin{center}\thud@address\end{center}%
                \fi%
            \fi%
        \end{minipage}%
    \end{titlepage}
    \thispagestyle{empty}%
    \setcounter{footnote}{0}
    \ifthud@phd%
        \ifx\thud@rights\@empty\else
            \vskip4em\relax\noindent\thud@rights\\\null\hrulefill%
        \fi%
        \vskip4em\relax%
        \let\and\\
        \thud@showentryB[\ttfamily]{homepage}%
        \thud@showentryB[\ttfamily]{email}%
        \thud@showentryB{telephone}%
        \thud@showentryB{address}%
        \vfill%
    \fi%
    \thud@spacing
    \let\thanks\relax\let\maketitle\relax
}

\AtBeginDocument{%
    \pagestyle{sfbig}%
    \frontmatter%
}
